@@ -1,37 +1,39 @@
 -- -*-sql-*- 
 --
 -- Copyright (C) 2008 Loic Dachary <loic@dachary.org>
--- Copyright (C) 2004, 2005, 2006 Mekensleep
+-- Copyright (C) 2004, 2005, 2006 Mekensleep <licensing@mekensleep.com>
+--                                24 rue vieille du temple, 75004 Paris
 --
--- Mekensleep
--- 24 rue vieille du temple
--- 75004 Paris
---       licensing@mekensleep.com
+-- This software's license gives you freedom; you can copy, convey,
+-- propagate, redistribute and/or modify this program under the terms of
+-- the GNU Affero General Public License (AGPL) as published by the Free
+-- Software Foundation, either version 3 of the License, or (at your
+-- option) any later version of the AGPL.
 --
--- This program is free software; you can redistribute it and/or modify
--- it under the terms of the GNU General Public License as published by
--- the Free Software Foundation; either version 3 of the License, or
--- (at your option) any later version.
+-- This program is distributed in the hope that it will be useful, but
+-- WITHOUT ANY WARRANTY; without even the implied warranty of
+-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
+-- General Public License for more details.
 --
--- This program is distributed in the hope that it will be useful,
--- but WITHOUT ANY WARRANTY; without even the implied warranty of
--- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--- GNU General Public License for more details.
---
--- You should have received a copy of the GNU General Public License
--- along with this program; if not, write to the Free Software
--- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+-- You should have received a copy of the GNU Affero General Public License
+-- along with this program in a file in the toplevel directory called
+-- "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
 --
 -- Authors:
 --  Loic Dachary <loic@gnu.org>
 --
+--
+-- NOTE: Be careful when you change the default tourney parameters in this
+-- file!  Some of the tests, most notably, test-pokerservice.py.in, use
+-- this schema, and make assumptions about the tourney setup.
+
 DROP TABLE IF EXISTS server;
 
 CREATE TABLE server (
   version VARCHAR(16) NOT NULL
 ) ENGINE=InnoDB CHARSET=utf8;
 
-INSERT INTO server (version) VALUES ("1.6.0");
+INSERT INTO server (version) VALUES ("1.7.4");
 
 DROP TABLE IF EXISTS users;
 
@@ -46,7 +48,7 @@ CREATE TABLE users (
   -- serial of the users record to which this user is affiliated
   affiliate INT UNSIGNED DEFAULT 0,
   -- client interpreted URL from which the player avatar can be downloaded
-  skin_url VARCHAR(32) DEFAULT "random",
+  skin_url VARCHAR(255) DEFAULT "random",
   -- client interpreted textual description of the player aspect
   skin_outfit TEXT,
   -- base64 encoded player picture of type skin_image_type
@@ -57,13 +59,15 @@ CREATE TABLE users (
   password VARCHAR(32),
   -- 1 is a regular player, 0 is an observer and cannot play, 2 is admin
   privilege INT DEFAULT 1,
+  -- locale
+  locale VARCHAR(32) DEFAULT "en_US",
 
   rating INT DEFAULT 1000,
   future_rating FLOAT DEFAULT 1000,
   games_count INT DEFAULT 0,
   
   PRIMARY KEY (serial),
-  UNIQUE KEY name_idx (name),
+  KEY name_idx (name),
   UNIQUE KEY email_idx (email)
 ) ENGINE=InnoDB CHARSET=utf8;
 
@@ -153,10 +157,27 @@ DROP TABLE IF EXISTS pokertables;
 
 CREATE TABLE pokertables (
   serial INT UNSIGNED NOT NULL AUTO_INCREMENT,
-  name VARCHAR(32),
+  resthost_serial INT UNSIGNED DEFAULT 0 NOT NULL,
+  seats TINYINT DEFAULT 10,
+  average_pot INT UNSIGNED DEFAULT 0,
+  hands_per_hour INT UNSIGNED DEFAULT 0,
+  percent_flop TINYINT DEFAULT 0,
+  players TINYINT DEFAULT 0,
+  observers INT UNSIGNED DEFAULT 0,
+  waiting TINYINT DEFAULT 0,
+  player_timeout INT UNSIGNED DEFAULT 60,
+  muck_timeout INT UNSIGNED DEFAULT 5,
   currency_serial INT NOT NULL,
+  name VARCHAR(255) NOT NULL UNIQUE,
+  variant VARCHAR(255) NOT NULL,
+  betting_structure VARCHAR(255) NOT NULL,
+  skin VARCHAR(255) DEFAULT "default" NOT NULL,
+  tourney_serial INT UNSIGNED DEFAULT 0 NOT NULL,
 
-  PRIMARY KEY (serial)
+  PRIMARY KEY (serial),
+  KEY (name),
+  KEY (variant),
+  KEY (resthost_serial)
 )  ENGINE=InnoDB CHARSET=utf8;
 
 DROP TABLE IF EXISTS hands;
@@ -205,7 +226,8 @@ DROP TABLE IF EXISTS tourneys_schedule;
 
 CREATE TABLE tourneys_schedule (
   serial INT UNSIGNED NOT NULL AUTO_INCREMENT,
-  name VARCHAR(32),
+  resthost_serial INT UNSIGNED DEFAULT 0 NOT NULL,
+  name VARCHAR(200),
   -- Description that will be shown in tournament lists
   description_short VARCHAR(64),
   -- Description that will be shown on a detailed page about
@@ -264,6 +286,10 @@ CREATE TABLE tourneys_schedule (
   respawn CHAR DEFAULT 'n',
   -- Unused.
   respawn_interval INT DEFAULT 0,
+  -- Format string to override currency serial from date 
+  -- The string can only contain %Y (2008) %m (03) or %d (07)
+  -- Example: '%Y%m' 
+  currency_serial_from_date_format VARCHAR(16) DEFAULT NULL,
 
   PRIMARY KEY (serial),
   KEY tourneys_schedule_active_index (active),
@@ -274,6 +300,7 @@ CREATE TABLE tourneys_schedule (
 --
 -- Holdem sit and go
 --
+-- SEE NOTE ABOVE BEFORE CHANGING!!!!
 INSERT INTO `tourneys_schedule` ( `name`, `description_short` , `description_long` , `players_quota` , `variant` , `betting_structure` , `seats_per_game` , `currency_serial` , `buy_in` , `rake` , `sit_n_go` , `start_time` , `register_time` , `respawn` , `respawn_interval` )
 VALUES ( 'sitngo2', 'Sit and Go 2 players, Holdem', 'Sit and Go 2 players', '2', 'holdem', 'level-15-30-no-limit', '2', 1, '300000', '0', 'y', '0', '0', 'y', '0' );
 
@@ -348,13 +375,15 @@ VALUES ( 'sitngo2', 'Sit and Go 2 players, Holdem', 'Sit and Go 2 players', '2',
 -- INSERT INTO `tourneys_schedule` ( `name`, `description_short` , `description_long` , `players_quota` , `variant` , `betting_structure` , `seats_per_game` , `currency_serial` , `buy_in` , `rake` , `sit_n_go` , `start_time` , `register_time` , `respawn` , `respawn_interval` )
 -- VALUES ( 'Ositngo50', 'Sit and Go 5 tables, Omaha', 'Sit and Go 5 tables', '50', 'omaha', 'level-15-30-no-limit', '10', 1, '300000', '0', 'y', '0', '0', 'y', '0' );
 
-INSERT INTO `tourneys_schedule` (`serial`, `name`, `description_short`, `description_long`, `players_quota`, `variant`, `betting_structure`, `seats_per_game`, `currency_serial`, `buy_in`, `rake`, `sit_n_go`, `breaks_interval`, `rebuy_delay`, `add_on`, `add_on_delay`, `start_time`, `register_time`, `respawn`, `respawn_interval`, `players_min`) VALUES ('', 'regular1', 'Holdem No Limit Freeroll', 'Holdem No Limit Freeroll', '1000', 'holdem', 'level-001', '10', 1, '0', '0', 'n', '60', '30', '1', '60', unix_timestamp(now() + INTERVAL 2 MINUTE), unix_timestamp(now()), 'y', '0', 3);
+-- SEE NOTE ABOVE BEFORE CHANGING!!!!
+INSERT INTO `tourneys_schedule` (`serial`, `name`, `description_short`, `description_long`, `players_quota`, `variant`, `betting_structure`, `seats_per_game`, `currency_serial`, `buy_in`, `rake`, `sit_n_go`, `breaks_interval`, `rebuy_delay`, `add_on`, `add_on_delay`, `start_time`, `register_time`, `respawn`, `respawn_interval`, `players_min`) VALUES ('', 'regular1', 'Holdem No Limit Freeroll', 'Holdem No Limit Freeroll', '1000', 'holdem', 'level-001', '10', 1, '0', '0', 'n', '60', '30', '1', '60', unix_timestamp(now() + INTERVAL 2 MINUTE), unix_timestamp(now() - INTERVAL 1 HOUR), 'n', '0', 3);
 
 DROP TABLE IF EXISTS tourneys;
 
 CREATE TABLE tourneys (
   serial INT UNSIGNED NOT NULL AUTO_INCREMENT,
-  name VARCHAR(32),
+  resthost_serial INT UNSIGNED DEFAULT 0 NOT NULL,
+  name VARCHAR(200),
   description_short VARCHAR(64),
   description_long TEXT,
   players_quota INT DEFAULT 10,
@@ -378,25 +407,29 @@ CREATE TABLE tourneys (
   start_time INT DEFAULT 0,
 
   finish_time INT DEFAULT 0,
-  state VARCHAR(32) DEFAULT "registering",
+  state VARCHAR(16) DEFAULT "registering",
   schedule_serial INT,
   add_on_count INT DEFAULT 0,
   rebuy_count INT DEFAULT 0,
 
   PRIMARY KEY (serial),
-  KEY tourneys_start_time_index (start_time)
-
+  KEY tourneys_start_time_index (start_time),
+  KEY state (state, finish_time)
 )  ENGINE=InnoDB CHARSET=utf8;
 
 DROP TABLE IF EXISTS user2tourney;
 
 CREATE TABLE user2tourney (
   user_serial INT NOT NULL,
+  currency_serial INT UNSIGNED NOT NULL,
   tourney_serial INT NOT NULL,
   table_serial INT,
   rank INT DEFAULT -1,
 
-  PRIMARY KEY (user_serial, tourney_serial)
+  PRIMARY KEY (user_serial, tourney_serial),
+  KEY user_serial (user_serial, currency_serial, tourney_serial),
+  KEY rank (rank),
+  KEY tourney_serial (tourney_serial)
 )  ENGINE=InnoDB CHARSET=utf8;
 
 --
@@ -471,3 +504,57 @@ CREATE TABLE messages (
 
   PRIMARY KEY (serial, send_date, sent)
 ) ENGINE=InnoDB CHARSET=utf8;
+
+DROP TABLE IF EXISTS monitor;
+
+CREATE TABLE monitor (
+  -- primary key
+  serial BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
+  -- creation date
+  created TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
+  -- for more information about the following, see
+  -- pokerpackets.py PacketPokerMonitorEvent documentation
+  event TINYINT NOT NULL,
+  param1 BIGINT NOT NULL,
+  param2 BIGINT NOT NULL,
+  param3 BIGINT NOT NULL,
+
+  PRIMARY KEY (serial)
+) ENGINE=MyISAM;
+
+DROP TABLE IF EXISTS resthost;
+
+CREATE TABLE resthost (
+  serial INT UNSIGNED NOT NULL AUTO_INCREMENT,
+  name VARCHAR(255),
+  host VARCHAR(255),
+  port INT UNSIGNED,
+  path VARCHAR(255),
+
+  PRIMARY KEY (serial)
+) ENGINE=MEMORY;
+
+DROP TABLE IF EXISTS route;
+
+CREATE TABLE route (
+  table_serial INT UNSIGNED,
+  tourney_serial INT UNSIGNED,
+  modified INT UNSIGNED,
+  resthost_serial INT UNSIGNED,
+  
+  PRIMARY KEY (table_serial, tourney_serial),
+  KEY (table_serial),
+  KEY (tourney_serial),
+  KEY (modified),
+  KEY (resthost_serial)
+) ENGINE=MEMORY;
+
+-- CREATE TABLE lastpoll (
+--   session CHAR(32) NOT NULL,
+--   user_serial INT UNSIGNED NOT NULL,
+--   table_serial INT UNSIGNED,
+--   tourney_serial INT UNSIGNED,
+--   polled INT UNSIGNED,
+  
+--   PRIMARY KEY (session, user_serial, table_serial, tourney_serial)
+-- ) ENGINE=MEMORY;
