@@ -2,42 +2,46 @@
 # -*- coding: iso-8859-1 -*-
 #
 # Copyright (C) 2006, 2007, 2008 Loic Dachary <loic@dachary.org>
-# Copyright (C) 2008 Bradley M. Kuhn <bkuhn@ebb.org>
+# Copyright (C)             2008 Bradley M. Kuhn <bkuhn@ebb.org>
+# Copyright (C)             2008 Johan Euphrosine <proppy@aminche.com>
 # Copyright (C) 2004, 2005, 2006 Mekensleep
+#                                24 rue vieille du temple 75004 Paris
+#                                <licensing@mekensleep.com>
 #
-# Mekensleep
-# 24 rue vieille du temple
-# 75004 Paris
-#       licensing@mekensleep.com
+# This software's license gives you freedom; you can copy, convey,
+# propagate, redistribute and/or modify this program under the terms of
+# the GNU Affero General Public License (AGPL) as published by the Free
+# Software Foundation (FSF), either version 3 of the License, or (at your
+# option) any later version of the AGPL published by the FSF.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
+# General Public License for more details.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+# You should have received a copy of the GNU Affero General Public License
+# along with this program in a file in the toplevel directory called
+# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
 #
 # Authors:
 #  Loic Dachary <loic@gnu.org>
+#  Bradley M. Kuhn <bkuhn@ebb.org> (2008)
+#  Johan Euphrosine <proppy@aminche.com> (2008)
 #  Henry Precheur <henry@precheur.org> (2004)
-#
 
 from string import join
 import sets
 
 from twisted.internet import defer
 
+from types import *
+from traceback import format_exc
+
 from pokerengine import pokergame
 from pokernetwork.user import User, checkNameAndPassword
 from pokernetwork.pokerpackets import *
 from pokernetwork.pokerexplain import PokerExplain
+from twisted.internet import protocol, reactor, defer
 
 DEFAULT_PLAYER_USER_DATA = { 'ready': True }
 
@@ -45,11 +49,14 @@ class PokerAvatar:
 
     def __init__(self, service):
         self.protocol = None
+        self.localeFunc = None
         self.roles = sets.Set()
         self.service = service
         self.tables = {}
         self.user = User()
         self._packets_queue = []
+        self.warnedPacketExcess = False
+        self.tourneys = []
         self.setExplain(0)
         self.has_session = False
         self.bugous_processing_hand = False
@@ -66,11 +73,37 @@ class PokerAvatar:
                     return False
 
                 self.explain = PokerExplain(dirs = self.service.dirs,
-                                            verbose = self.service.verbose)
+                                            verbose = self.service.verbose,
+                                            explain = what)
         else:
             self.explain = None
         return True
+
+    def _setDefaultLocale(self, locale):
+        """Set self.localFunc using locale iff. it is not already set.
+        Typically, this method is only used for a locale found for the
+        user in the database.  If the client sends a
+        PacketPokerSetLocale(), that will always take precedent and should
+        not use this method, but self.setLocale() instead."""
+        if not self.localeFunc:
+            return self.setLocale(locale)
+        else:
+            return None
             
+    def setLocale(self, locale):
+        if locale:
+            # 'ISO-8859-1' is currently enforced for all setLocale()
+            # requests.  This is primarily because the JSON interface
+            # implemented in pokersite.py *assumes* that all strings must
+            # be 'ISO-8859-1' and encodes them to unicode.  We should
+            # actually find a way to be knowledgeable about the needed
+            # encoding, because there is probably pointless conversion
+            # between ISO and UTF-8 happening in this process.  Also, we
+            # cannot currently support any languages that are non
+            # 'ISO-8859-1', so that's a huge FIXME!  --bkuhn, 2008-10-28
+            self.localeFunc = self.service.locale2translationFunc(locale, 'ISO-8859-1')
+        return self.localeFunc
+
     def setProtocol(self, protocol):
         self.protocol = protocol
 
@@ -86,6 +119,23 @@ class PokerAvatar:
     def isAuthorized(self, type):
         return self.user.hasPrivilege(self.service.poker_auth.GetLevel(type))
 
+    def relogin(self, serial):
+        player_info = self.service.getPlayerInfo(serial)
+        self.user.serial = serial
+        self.user.name = player_info.name
+        self.user.privilege = User.REGULAR
+        self.user.url = player_info.url
+        self.user.outfit = player_info.outfit
+        self._setDefaultLocale(player_info.locale)
+
+        if self.explain:
+            self.explain.handleSerial(PacketSerial(serial = serial))
+        if self.service.serial2client.has_key(serial):
+            self.service.destroyAvatar(self.service.serial2client[serial])
+        self.service.serial2client[serial] = self
+        self.tourneyUpdates(serial)
+        self.loginTableUpdates(serial)
+    
     def login(self, info):
         (serial, name, privilege) = info
         self.user.serial = serial
@@ -95,7 +145,8 @@ class PokerAvatar:
         player_info = self.service.getPlayerInfo(serial)
         self.user.url = player_info.url
         self.user.outfit = player_info.outfit
-        
+        self._setDefaultLocale(player_info.locale)
+
         self.sendPacketVerbose(PacketSerial(serial = self.user.serial))
         if PacketPokerRoles.PLAY in self.roles:
             self.service.serial2client[serial] = self
@@ -103,6 +154,14 @@ class PokerAvatar:
             self.message("user %s/%d logged in" % ( self.user.name, self.user.serial ))
         if self.protocol:
             self.has_session = self.service.sessionStart(self.getSerial(), str(self.protocol.transport.client[0]))
+        self.tourneyUpdates(serial)
+        self.loginTableUpdates(serial)
+
+    def tourneyUpdates(self, serial):
+        places = self.service.getPlayerPlaces(serial)
+        self.tourneys = places.tourneys
+
+    def loginTableUpdates(self, serial):
         #
         # Send player updates if it turns out that the player was already
         # seated at a known table.
@@ -177,23 +236,94 @@ class PokerAvatar:
     def noqueuePackets(self):
         self._queue_packets = False
 
+    def extendPacketsQueue(self, newPackets):
+        """takes PokerAvatar object and a newPackets as arguments, and
+        extends the self._queue_packets variable by that packet.  Checking
+        is done to make sure we haven't exceeded server-wide limits on
+        packet queue length.  PokerAvatar will be force-disconnected if
+        the packets exceed the value of
+        self.service.getClientQueuedPacketMax().  A warning will be
+        printed when the packet queue reaches 75% of the limit imposed by
+        self.service.getClientQueuedPacketMax()"""
+        # This method was introduced when we added the force-disconnect as
+        # the stop-gap.
+        self._packets_queue.extend(newPackets)
+        warnVal = int(.75 * self.service.getClientQueuedPacketMax())
+        if len(self._packets_queue) >= warnVal:
+            # If we have not warned yet that packet queue is getting long, warn now.
+            if not self.warnedPacketExcess:
+                self.warnedPacketExcess = True
+                self.error("WARNING: user %d has more than %d packets queued; will force-disconnect when %d are queued" % (self.getSerial(), warnVal, self.service.getClientQueuedPacketMax()))
+            if len(self._packets_queue) >= self.service.getClientQueuedPacketMax():
+                self.service.forceAvatarDestroy(self)
+
     def resetPacketsQueue(self):
+        self.warnedPacketExcess = False
         queue = self._packets_queue
         self._packets_queue = []
         return queue
     
     def sendPacket(self, packet):
+        from pokerengine.pokergame import init_i18n as pokergame_init_i18n
+        # Note on special processing of locales on packet send:
+        #    Ideally, clients would do their own locale work.  However, in
+        #    particular when PokerExplain is in effect, some clients are
+        #    requiring explanation strings coming from the server about
+        #    what is happening in the game.  (Indeed, PokerExplain exists
+        #    for precisely that scenario.)  Therefore, every time we send a
+        #    packet via PokerAvatar, we need to make sure the local in
+        #    poker-engine's pokergame localization is set properly to the
+        #    localization requested by the client (iff. they have
+        #    requested one via PacketPokerSetLocale).  Note that because
+        #    global variables are effectively only file-wide, the _() that
+        #    we create here propagates only as wide as this file.  the
+        #    call to pokergame_init_i18n() is what actually changes the
+        #    _() defined in pokergame.py.
+        #
+        #    It is in some ways overkill to redefine our own _() here,
+        #    particularly because at the time of writing, we don't
+        #    actually have localization strings in the functions in this
+        #    file.  However, should we have them later, we'd obviously
+        #    want those strings to be localized for the client, at least
+        #    during packet sending.
+
+        global _
+        if self.localeFunc:
+            avatarSavedUnder = _
+            _ = self.localeFunc
+            pokergameSavedUnder = pokergame_init_i18n('', self.localeFunc)
 	if self.explain and not isinstance(packet, defer.Deferred) and packet.type != PACKET_ERROR:
-	    self.explain.explain(packet)
-	    packets = self.explain.forward_packets
+            try:
+                self.explain.explain(packet)
+                packets = self.explain.forward_packets
+            except:
+                packets = [ PacketError(other_type = PACKET_NONE, message = format_exc()) ]
+                if self.service.verbose >= 0:
+                    self.message(packets[0].message)
+                self.explain = None # disabling the explain instance
+                                    # that issued the exception, as it
+                                    # may be in an inconsistent state,
+                                    # and used before the avatar
+                                    # destruction
+                self.service.forceAvatarDestroy(self)
 	else:
 	    packets = [ packet ]
         if self._queue_packets:
-            self._packets_queue.extend(packets)
+            self.extendPacketsQueue(packets)
         else:
 	    for packet in packets:
                 self.protocol.sendPacket(packet)
+        if self.localeFunc:
+            _ = avatarSavedUnder
+            pokergame_init_i18n('', pokergameSavedUnder)
 
+    # Below, we assign the method queueDeferred() is the same as
+    # sendPacket().  Be careful not to indent the line below; if you
+    # aren't paying attention, you might think it belongs inside the
+    # previous function.  It doesn't. ...  Ok, so I never got over the
+    # "whitespace indentation matters" thing in Python, and I get careless
+    # sometimes, then after I do I proceed to write warning comments that
+    # normal Python programmers probably don't need. :-p -- bkuhn
     queueDeferred = sendPacket
     
     def sendPacketVerbose(self, packet):
@@ -207,12 +337,31 @@ class PokerAvatar:
         else:
             return False
 
+    def handlePacketDefer(self, packet):
+        self.queuePackets()
+        self.handlePacketLogic(packet)
+        packets = self.resetPacketsQueue()
+        if len(packets) == 1 and isinstance(packets[0], defer.Deferred):
+            d = packets[0]
+            #
+            # turn the return value into an List if it is not
+            #
+            def packetList(result):
+                if type(result) == ListType:
+                    return result
+                else:
+                    return [ result ]
+            d.addCallback(packetList)
+            return d
+        else:
+            return packets
+
     def handlePacket(self, packet):
         self.queuePackets()
         self.handlePacketLogic(packet)
         self.noqueuePackets()
         return self.resetPacketsQueue()
-        
+
     def handlePacketLogic(self, packet):
         if self.service.verbose > 2 and packet.type != PACKET_PING:
             self.message("handlePacketLogic(%d): " % self.getSerial() + str(packet))
@@ -224,17 +373,34 @@ class PokerAvatar:
                 self.sendPacketVerbose(PacketError(other_type = PACKET_POKER_EXPLAIN))
             return
         
+        if packet.type == PACKET_POKER_SET_LOCALE:
+            if self.setLocale(packet.locale):
+                self.sendPacketVerbose(PacketAck())
+            else:
+                self.sendPacketVerbose(PacketPokerError(serial = self.getSerial(),
+                                                        other_type = PACKET_POKER_SET_LOCALE))
+            return
+
         if packet.type == PACKET_POKER_STATS_QUERY:
             self.sendPacketVerbose(self.service.stats(packet.string))
             return
         
+        if packet.type == PACKET_POKER_MONITOR:
+            self.sendPacketVerbose(self.service.monitor(self))
+            return
+        
+        if packet.type == PACKET_PING:
+            return
+        
+        if packet.type == PACKET_POKER_POLL:
+            if packet.tourney_serial != 0 and not (packet.tourney_serial in self.tourneys):
+                self.sendPacketVerbose(PacketPokerTourneyFinish(tourney_serial = packet.tourney_serial))
+            return
+        
         if not self.isAuthorized(packet.type):
             self.sendPacketVerbose(PacketAuthRequest())
             return
 
-        if packet.type == PACKET_PING:
-            return
-        
         if packet.type == PACKET_LOGIN:
             if self.isLogged():
                 self.sendPacketVerbose(PacketError(other_type = PACKET_LOGIN,
@@ -244,6 +410,13 @@ class PokerAvatar:
                 self.auth(packet)
             return
 
+        if packet.type == PACKET_POKER_GET_PLAYER_PLACES:
+            if packet.serial != 0:
+                self.sendPacketVerbose(self.service.getPlayerPlaces(packet.serial))
+            else:
+                self.sendPacketVerbose(self.service.getPlayerPlacesByName(packet.name))
+            return
+
         if packet.type == PACKET_POKER_GET_PLAYER_INFO:
             self.sendPacketVerbose(self.getPlayerInfo())
             return
@@ -336,21 +509,31 @@ class PokerAvatar:
             return
 
         if packet.type == PACKET_POKER_TOURNEY_SELECT:
-            ( players, tourneys ) = self.service.tourneyStats()
-            tourneys = PacketPokerTourneyList(players = players,
-                                              tourneys = tourneys)
-            for tourney in self.service.tourneySelect(packet.string):
-                tourneys.packets.append(PacketPokerTourney(**tourney))
-            self.sendPacketVerbose(tourneys)
+            ( playerCount, tourneyCount ) = self.service.tourneyStats()
+            tourneyList = PacketPokerTourneyList(players = playerCount,
+                                              tourneys = tourneyCount)
+            tourneys = self.service.tourneySelect(packet.string)
+            for tourney in tourneys:
+                tourneyList.packets.append(PacketPokerTourney(**tourney))
+            self.sendPacketVerbose(tourneyList)
+            tourneyInfo = self.service.tourneySelectInfo(packet, tourneys)
+            if tourneyInfo:
+                self.sendPacketVerbose(tourneyInfo)
             return
         
         elif packet.type == PACKET_POKER_TOURNEY_REQUEST_PLAYERS_LIST:
             self.sendPacketVerbose(self.service.tourneyPlayersList(packet.game_id))
             return
 
+        elif packet.type == PACKET_POKER_GET_TOURNEY_MANAGER:
+            self.sendPacketVerbose(self.service.tourneyManager(packet.tourney_serial))
+            return
+
         elif packet.type == PACKET_POKER_TOURNEY_REGISTER:
             if self.getSerial() == packet.serial:
+                self.service.autorefill(packet.serial)
                 self.service.tourneyRegister(packet)
+                self.tourneyUpdates(packet.serial)
             else:
                 self.message("attempt to register in tournament %d for player %d by player %d" % ( packet.game_id, packet.serial, self.getSerial() ))
             return
@@ -358,6 +541,7 @@ class PokerAvatar:
         elif packet.type == PACKET_POKER_TOURNEY_UNREGISTER:
             if self.getSerial() == packet.serial:
                 self.sendPacketVerbose(self.service.tourneyUnregister(packet))
+                self.tourneyUpdates(packet.serial)
             else:
                 self.message("attempt to unregister from tournament %d for player %d by player %d" % ( packet.game_id, packet.serial, self.getSerial() ))
             return
@@ -400,7 +584,10 @@ class PokerAvatar:
                 self.message("packet for table " + str(table.game.id))
             game = table.game
 
-            if packet.type == PACKET_POKER_READY_TO_PLAY:
+            if packet.type == PACKET_POKER_GET_PLAYER_STATS:
+                self.sendPacketVerbose(self.getPlayerStats(table, packet.serial))
+
+            elif packet.type == PACKET_POKER_READY_TO_PLAY:
                 if self.getSerial() == packet.serial:
                     self.sendPacketVerbose(table.readyToPlay(packet.serial))
                 else:
@@ -418,7 +605,7 @@ class PokerAvatar:
                     self.message("attempt to set processing hand for player %d by player %d that is not the owner of the game" % ( packet.serial, self.getSerial() ))
 
             elif packet.type == PACKET_POKER_START:
-                if game.isEndOrNull():
+                if not game.isEndOrNull():
                     self.message("player %d tried to start a new game while in game " % self.getSerial())
                     self.sendPacketVerbose(PacketPokerStart(game_id = game.id))
                 elif self.service.shutting_down:
@@ -433,7 +620,13 @@ class PokerAvatar:
                     self.message("player %d tried to start a new game but is not the owner of the table" % self.getSerial())
 
             elif packet.type == PACKET_POKER_SEAT:
-                if ( self.getSerial() == packet.serial or
+                if PacketPokerRoles.PLAY not in self.roles:
+                    self.sendPacketVerbose(PacketPokerError(game_id = game.id,
+                                                            serial = packet.serial,
+                                                            code = PacketPokerSeat.ROLE_PLAY,
+                                                            message = "PACKET_POKER_ROLES must set the role to PLAY before chosing a seat",
+                                                            other_type = PACKET_POKER_SEAT))
+                elif ( self.getSerial() == packet.serial or
                      self.getSerial() == table.owner ):
                     if not table.seatPlayer(self, packet.serial, packet.seat):
                         packet.seat = -1
@@ -446,6 +639,7 @@ class PokerAvatar:
                 
             elif packet.type == PACKET_POKER_BUY_IN:
                 if self.getSerial() == packet.serial:
+                    self.service.autorefill(packet.serial)
                     if not table.buyInPlayer(self, packet.amount):
                         self.sendPacketVerbose(PacketPokerError(game_id = game.id,
                                                                 serial = packet.serial,
@@ -455,6 +649,7 @@ class PokerAvatar:
 
             elif packet.type == PACKET_POKER_REBUY:
                 if self.getSerial() == packet.serial:
+                    self.service.autorefill(packet.serial)
                     if not table.rebuyPlayerRequest(self, packet.amount):
                         self.sendPacketVerbose(PacketPokerError(game_id = game.id,
                                                                 serial = packet.serial,
@@ -633,6 +828,14 @@ class PokerAvatar:
                                message = "Not logged in",
                                other_type = PACKET_POKER_GET_PLAYER_INFO)
     
+    def getPlayerStats(self, table, serialSought):
+        if self.user.isLogged():
+            return self.service.getUserStatsLookup().getAttrsAsPacket(table = table, serial = serialSought)
+        else:
+            return PacketError(code = PacketPokerGetPlayerStats.NOT_LOGGED,
+                               message = "Not logged in",
+                               other_type = PACKET_POKER_GET_PLAYER_STATS)
+
     def listPlayers(self, packet):
         table = self.service.getTable(packet.game_id)
         if table:
@@ -643,7 +846,24 @@ class PokerAvatar:
     def listTables(self, packet):
         packets = []
         for table in self.service.listTables(packet.string, self.getSerial()):
-            packets.append(table.toPacket())
+            packet = PacketPokerTable(id = int(table['serial']),
+                                      name = table['name'],
+                                      variant = table['variant'],
+                                      betting_structure = table['betting_structure'],
+                                      seats = int(table['seats']),
+                                      players = int(table['players']),
+                                      hands_per_hour = int(table['hands_per_hour']),
+                                      average_pot = int(table['average_pot']),
+                                      percent_flop = int(table['percent_flop']),
+                                      player_timeout = int(table['player_timeout']),
+                                      muck_timeout = int(table['muck_timeout']),
+                                      observers = int(table['observers']),
+                                      waiting = int(table['waiting']),
+                                      skin = table['skin'],
+                                      currency_serial = int(table['currency_serial']),
+                                      )
+            packet.tourney_serial = int(table['tourney_serial'])
+            packets.append(packet)
         ( players, tables ) = self.service.statsTables()
         self.sendPacketVerbose(PacketPokerTableList(players = players,
                                                     tables = tables,
@@ -688,8 +908,9 @@ class PokerAvatar:
             "variant": packet.variant,
             "betting_structure": packet.betting_structure,
             "player_timeout": packet.player_timeout,
+            "muck_timeout": packet.muck_timeout,
             "currency_serial": packet.currency_serial,
-            "transient": True })
+            "skin": packet.skin })
         if not table:
             self.sendPacket(PacketPokerTable())
         return table            
@@ -722,6 +943,8 @@ class PokerAvatar:
                                                            auto_blind_ante = player.auto_blind_ante,
                                                            wait_for = player.wait_for,
                                                            seat = player.seat))
+            # Send the stats packet, generated by the UserStats package.
+            self.sendPacketVerbose(self.service.getUserStatsLookup().getAttrsAsPacket(table = table, serial = player.serial))
             if not game.isPlaying(player.serial):
                 self.sendPacketVerbose(PacketPokerPlayerChips(game_id = game.id,
                                                               serial = player.serial,
@@ -730,6 +953,9 @@ class PokerAvatar:
                 if game.isSit(player.serial):
                     self.sendPacketVerbose(PacketPokerSit(game_id = game.id,
                                                           serial = player.serial))
+                if player.isAuto():
+                    self.sendPacketVerbose(PacketPokerAutoFold(game_id = game.id,
+                                                          serial = player.serial))                    
 
         self.sendPacketVerbose(PacketPokerSeats(game_id = game.id,
                                                 seats = game.seats()))
@@ -756,16 +982,18 @@ class PokerAvatar:
         table.sendNewPlayerInformation(serial)
         
     def connectionLost(self, reason):
-        if self.service.verbose:
+        if self.service.verbose > 3:
             self.message("connection lost for %s/%d" % ( self.getName(), self.getSerial() ))
         for table in self.tables.values():
             table.disconnectPlayer(self, self.getSerial())
         self.logout()
 
     def getUserInfo(self, serial):
+        self.service.autorefill(serial)
         self.sendPacketVerbose(self.service.getUserInfo(serial))
 
     def getPersonalInfo(self, serial):
+        self.service.autorefill(serial)
         self.sendPacketVerbose(self.service.getPersonalInfo(serial))
 
     def removePlayer(self, table, serial):
