@@ -1,29 +1,26 @@
 # -*- py-indent-offset: 4; coding: iso-8859-1; mode: python -*-
 #
 # Copyright (C) 2006, 2007, 2008 Loic Dachary <loic@dachary.org>
-# Copyright (C) 2004, 2005, 2006 Mekensleep
+# Copyright (C) 2004, 2005, 2006 Mekensleep <licensing@mekensleep.com>
+#                                24 rue vieille du temple, 75004 Paris
 #
-# Mekensleep
-# 24 rue vieille du temple
-# 75004 Paris
-#       licensing@mekensleep.com
+# This software's license gives you freedom; you can copy, convey,
+# propagate, redistribute and/or modify this program under the terms of
+# the GNU Affero General Public License (AGPL) as published by the Free
+# Software Foundation (FSF), either version 3 of the License, or (at your
+# option) any later version of the AGPL published by the FSF.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
+# General Public License for more details.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+# You should have received a copy of the GNU Affero General Public License
+# along with this program in a file in the toplevel directory called
+# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
 #
 # Authors:
-#  Loic Dachary <loic@gnu.org>
+#  Loic Dachary <loic@dachary.org>
 #  Henry Precheur <henry@precheur.org> (2004)
 #
 
@@ -362,6 +359,7 @@ class PokerClientFactory(UGAMEClientFactory):
         if platform.system() == "Windows":
             os.execv(upgrades_dir + "/upgrade.exe", [ upgrades_dir + "/upgrade.exe", '"' + target_dir + '"', '"' + sys.executable + '"' ])
         else:
+            # FIXME: shouldn't this be an .in file and use @SHELL@ here?
             os.execv("/bin/sh", [ upgrades_dir + "/upgrade", '-x', upgrades_dir + "/upgrade", upgrades_dir, sys.executable ] + sys.argv)
 
 ABSOLUTE_LAGMAX = 120
@@ -760,6 +758,7 @@ class PokerClientProtocol(UGAMEClientProtocol):
                                                    auto_blind_ante = player.auto_blind_ante,
                                                    wait_for = player.wait_for,
                                                    seat = player.seat))
+            # FIXME: Should a PokerPlayerStats() packet be sent here?
             if player.isSit():
                 packets.append(PacketPokerSit(game_id = game.id,
                                               serial = player.serial))
@@ -842,6 +841,9 @@ class PokerClientProtocol(UGAMEClientProtocol):
 
     def protocolEstablished(self):
         self.setPingDelay(self.factory.ping_delay)
+        poll_frequency = self.factory.settings.headerGet("/settings/@poll_frequency")
+        if poll_frequency:
+            self._poll_frequency = float(poll_frequency)
         self.user.name = self.factory.name
         self.user.password = self.factory.password
         self._packet2id = self.packet2id
