@@ -2,19 +2,20 @@
 #
 # Copyright (C) 2007, 2008 Loic Dachary <loic@dachary.org>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# This software's license gives you freedom; you can copy, convey,
+# propagate, redistribute and/or modify this program under the terms of
+# the GNU Affero General Public License (AGPL) as published by the Free
+# Software Foundation (FSF), either version 3 of the License, or (at your
+# option) any later version of the AGPL published by the FSF.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
+# General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+# You should have received a copy of the GNU Affero General Public License
+# along with this program in a file in the toplevel directory called
+# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
 #
 from string import lower
 
@@ -77,7 +78,10 @@ class PokerGames:
 
     def getAll(self):
         return self.games.values()
-    
+
+class ListHintSubset(Exception):
+    pass
+
 class PokerExplain:
     """Expand poker packets for the caller when they know nothing about poker """
 
@@ -90,6 +94,7 @@ class PokerExplain:
         self._prefix = ""
         self.games = PokerGames(**kwargs)
         self.setVerbose(kwargs.get('verbose', 0))
+        self.what = kwargs.get("explain", PacketPokerExplain.ALL)
 
     def error(self, string):
         self.message("ERROR " + string)
@@ -148,6 +153,8 @@ class PokerExplain:
                                             chips = self.normalizeChips(game, chips))
         packets.append(packet)
         packets.append(self.updatePlayerChips(game, player))
+        if self.what & PacketPokerExplain.CHIPSTACKS:
+            packets.append(self.explainPlayerChips(game, player))
         return packets
 
     def chipsBet2Pot(self, game, player, bet, pot_index):
@@ -165,6 +172,8 @@ class PokerExplain:
                                          pot = pot_index)
         packets.append(packet)
         packets.append(self.updatePlayerChips(game, player))
+        if self.what & PacketPokerExplain.CHIPSTACKS:
+            packets.append(self.explainPlayerChips(game, player))
         return packets
         
     def chipsPot2Player(self, game, player, bet, pot_index, reason):
@@ -268,6 +277,8 @@ class PokerExplain:
             if packet.id == 0:
                 self.error("Too many open tables")
             else:
+                if self.games.getGame(packet.id):
+                    self.games.deleteGame(packet.id)
                 new_game = self.games.getOrCreateGame(packet.id)
                 new_game.prefix = self._prefix
                 new_game.name = packet.name
@@ -275,7 +286,6 @@ class PokerExplain:
                 new_game.setVariant(packet.variant)
                 new_game.setBettingStructure(packet.betting_structure)
                 new_game.setMaxPlayers(packet.seats)
-                new_game.reset()
                 new_game.registerCallback(self.gameEvent)
                 new_game.level_skin = packet.skin
                 new_game.currency_serial = packet.currency_serial
@@ -309,11 +319,22 @@ class PokerExplain:
                     game.setHandsCount(packet.hands_count)
                     game.setLevel(packet.level)
                     game.beginTurn(packet.hand_serial)
+                    if game.player_list_hint != None and game.player_list_hint != game.player_list:
+                        player_list_hint = set(game.player_list_hint)
+                        player_list = set(game.player_list)
+                        if not player_list_hint.issubset(player_list):
+                            raise ListHintSubset, "%s is not a subset of %s" % ( game.player_list_hint, game.player_list )
+                        if self.verbose >= 3:
+                            self.message("player_list_hint discards %s" % ( player_list_hint ^ player_list ))
+                        game.player_list = game.player_list_hint
+                    game.player_list_hint = None
                     game.position_info[PokerNetworkGameClient.POSITION_OBSOLETE] = True
                     if not self.no_display_packets:
                         forward_packets.append(PacketPokerBoardCards(game_id = game.id, serial = self.getSerial()))
                         for serial in game.player_list:
                             forward_packets.append(self.updatePlayerChips(game, game.serial2player[serial]))
+                            if self.what & PacketPokerExplain.CHIPSTACKS:
+                                forward_packets.append(self.explainPlayerChips(game, game.serial2player[serial]))
                         forward_packets.extend(self.updatePotsChips(game, []))
 
             elif packet.type == PACKET_POKER_CANCELED:
@@ -343,10 +364,10 @@ class PokerExplain:
             elif ( packet.type == PACKET_POKER_PLAYER_LEAVE or
                    packet.type == PACKET_POKER_TABLE_MOVE ) :
                 game.removePlayer(packet.serial)
-                forward_packets.remove(packet)
-                forward_packets.append(PacketPokerPlayerLeave(game_id = packet.game_id,
-                                                              serial = packet.serial,
-                                                              seat = packet.seat))
+                if packet.type == PACKET_POKER_TABLE_MOVE:
+                    forward_packets.append(PacketPokerPlayerLeave(game_id = packet.game_id,
+                                                                  serial = packet.serial,
+                                                                  seat = packet.seat))
                 if not self.no_display_packets:
                     self.forward_packets.append(PacketPokerSeats(game_id = game.id,
                                                                  seats = game.seats()))
@@ -405,15 +426,22 @@ class PokerExplain:
                 forward_packets.remove(packet)
 
             elif packet.type == PACKET_POKER_IN_GAME:
+                game.player_list_hint = packet.players[:]
                 for serial in game.serialsAll():
                     player = game.getPlayer(serial)
                     wait_for = player.wait_for
                     in_game = serial in packet.players 
                     if in_game or wait_for:
+                        auto = player.isAuto()
                         if not game.isSit(serial):
                             game.sit(serial)
                             forward_packets.append(PacketPokerSit(game_id = game.id,
                                                                   serial = serial))
+                        if auto:
+                            game.autoPlayer(serial)
+                            forward_packets.append(PacketPokerAutoFold(game_id = game.id,
+                                                                       serial = player.serial))
+
                         if wait_for:
                             if wait_for == True and not in_game and not game.isRunning():
                                 #
@@ -482,11 +510,9 @@ class PokerExplain:
                     #
                     player = game.getPlayer(packet.serial)
                     player.buy_in_payed = True
-                    chips = PacketPokerPlayerChips(game_id = game.id,
-                                                   serial = packet.serial,
-                                                   money = player.money,
-                                                   bet = player.bet)
-                    forward_packets.append(chips)
+                    forward_packets.append(self.updatePlayerChips(game, player))
+                    if self.what & PacketPokerExplain.CHIPSTACKS:
+                        forward_packets.append(self.explainPlayerChips(game, player))
 
             elif packet.type == PACKET_POKER_PLAYER_CHIPS:
                 player = game.getPlayer(packet.serial)
@@ -507,6 +533,8 @@ class PokerExplain:
                     player.money = packet.money
                     if player.money > 0:
                         player.buy_in_payed = True
+                if self.what & PacketPokerExplain.CHIPSTACKS:
+                    forward_packets.append(self.explainPlayerChips(game, player))
 
             elif packet.type == PACKET_POKER_FOLD:
                 game.fold(packet.serial)
@@ -652,7 +680,7 @@ class PokerExplain:
 
         packets.extend(self.updatePotsChips(game, game.getPots()))
         return packets
-        
+
     #
     # Should be move all bets back to players (for uncalled bets)
     # This is a border case we don't want to handle right now
@@ -696,6 +724,15 @@ class PokerExplain:
             games.remove(exclude)
         return PacketPokerCurrentGames(game_ids = games,
                                        count = len(games))
+
+    def explainPlayerChips(self, game, player):
+        packet = PacketPokerClientPlayerChips(game_id = game.id,
+                                              serial = player.serial,
+                                              bet = self.normalizeChips(game, player.bet),
+                                              money = self.normalizeChips(game, player.money) )
+        return packet
+                
+        
     
     def packetsPot2Player(self, game):
         packets = []
@@ -747,6 +784,8 @@ class PokerExplain:
                 
         for player in game.serial2player.itervalues():
             packets.append(self.updatePlayerChips(game, player))
+            if self.what & PacketPokerExplain.CHIPSTACKS:
+                packets.append(self.explainPlayerChips(game, player))
         packets.extend(self.updatePotsChips(game, []))
         return packets
         
