@@ -2,29 +2,28 @@
 # -*- coding: iso-8859-1 -*-
 #
 # Copyright (C) 2006, 2007, 2008 Loic Dachary <loic@dachary.org>
-# Copyright (C) 2004, 2005, 2006 Mekensleep
+# Copyright (C)             2008 Bradley M. Kuhn <bkuhn@ebb.org>
+# Copyright (C) 2004, 2005, 2006 Mekensleep <licensing@mekensleep.com>
+#                                24 rue vieille du temple 75004 Paris
 #
-# Mekensleep
-# 24 rue vieille du temple
-# 75004 Paris
-#       licensing@mekensleep.com
+# This software's license gives you freedom; you can copy, convey,
+# propagate, redistribute and/or modify this program under the terms of
+# the GNU Affero General Public License (AGPL) as published by the Free
+# Software Foundation (FSF), either version 3 of the License, or (at your
+# option) any later version of the AGPL published by the FSF.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
+# General Public License for more details.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
+# You should have received a copy of the GNU Affero General Public License
+# along with this program in a file in the toplevel directory called
+# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
 #
 # Authors:
-#  Loic Dachary <loic@gnu.org>
+#  Loic Dachary <loic@dachary.org>
+#  Bradley M. Kuhn <bkuhn@ebb.org> (2008-)
 #  Henry Precheur <henry@precheur.org> (2004)
 #
 
@@ -35,6 +34,7 @@ from re import match
 from types import *
 from string import split, join
 import time
+import traceback
 
 from pokerengine.pokergame import PokerGameServer, history2messages
 from pokerengine import pokergame, pokertournament
@@ -81,6 +81,12 @@ class PokerTable:
         self.muckTimeout = int(description.get("muck_timeout", 5))
         self.transient = description.has_key("transient")
         self.tourney = description.get("tourney", None)
+
+        # max_missed_round can be configured on a per table basis, which
+        # overrides the server-wide default
+        self.max_missed_round = description.get("max_missed_round",
+                                                factory.getMissedRoundMax())
+
         self.delays = settings.headerGetProperties("/server/delays")[0]
         self.autodeal = settings.headerGet("/server/@autodeal") == "yes"
         self.temporaryPlayersPattern = settings.headerGet("/server/users/@temporary")
@@ -97,7 +103,8 @@ class PokerTable:
         self.game_delay = {
             "start": 0,
             "delay": 0,
-            }            
+            }
+        self.update_recursion = False
 
     def message(self, string):
         print "PokerTable: " + string
@@ -134,7 +141,7 @@ class PokerTable:
         return name
 
     def getPlayerInfo(self, serial):
-        if self.serial2client.has_key(serial):
+        if self.serial2client.has_key(serial) and self.serial2client[serial].user.isLogged():
             info = self.serial2client[serial].getPlayerInfo()
         else:
             info = self.factory.getPlayerInfo(serial)
@@ -189,7 +196,8 @@ class PokerTable:
                                 observers = len(self.observers),
                                 waiting = len(self.waiting),
                                 skin = self.skin,
-                                currency_serial = self.currency_serial)
+                                currency_serial = self.currency_serial,
+                                tourney_serial = self.tourney and self.tourney.serial or 0)
                 
     def cards2packets(self, game_id, board, pockets, cache):
         packets = []
@@ -236,6 +244,8 @@ class PokerTable:
             for client in self.observers:
                 client.sendPacket(self.private2public(packet, 0))
 
+        self.factory.eventTable(self)
+
     def private2public(self, packet, serial):
         game = self.game
         #
@@ -515,6 +525,9 @@ class PokerTable:
             elif type == "finish":
                 (type, hand_serial) = event
                 self.factory.saveHand(self.compressedHistory(game.historyGet()), hand_serial)
+                self.factory.updateTableStats(game, len(self.observers), len(self.waiting))
+                transient = self.transient and 1 or 0
+                self.factory.databaseEvent(event = PacketPokerMonitorEvent.HAND, param1 = hand_serial, param2 = transient)
             
             else:
                 self.error("syncDatabase: unknown history type %s " % type)
@@ -642,6 +655,22 @@ class PokerTable:
                     if self.serial2client.has_key(serial):
                         self.seated2observer(self.serial2client[serial])
 
+    def cashGame_kickPlayerSittingOutTooLong(self, historyToSearch):
+        if self.tourney: return
+        handIsFinished = False
+        # Go through the history backwards, stopping at
+        # self.history_index, since we expect finish to be at the end if
+        # it is there, and we don't want to consider previously reduced
+        # history.
+        for event in historyToSearch[::-1]:
+            if event[0] == "finish":
+                handIsFinished = True
+                break
+        if handIsFinished:
+            for player in self.game.playersAll():
+                if player.getMissedRoundCount() >= self.max_missed_round:
+                    self.kickPlayer(player.serial)
+
     def tourneyEndTurn(self):
         if not self.tourney:
             return
@@ -764,16 +793,9 @@ class PokerTable:
             if delta > autodeal_max: delta = autodeal_max
             self.game_delay["delay"] = ( seconds() - self.game_delay["start"] ) + delta
         elif self.transient:
-            all_auto = True
-            for player in game.playersAll():
-                if not player.isAuto():
-                    all_auto = False
-                    delta = 0
-                    break
-            if all_auto:
-                delta = int(self.delays.get("autodeal_tournament_min", 15))
-                if seconds() - self.game_delay["start"] > delta:
-                    delta = 0
+            delta = int(self.delays.get("autodeal_tournament_min", 15))
+            if seconds() - self.game_delay["start"] > delta:
+                delta = 0
         else:
             delta = 0
         if self.factory.verbose > 2:
@@ -812,22 +834,32 @@ class PokerTable:
         return PacketAck()
         
     def update(self):
+        if self.update_recursion:
+            if self.factory.verbose >= 0:
+                self.error("unexpected recursion (ignored)\n" + "".join(traceback.format_list(traceback.extract_stack())))
+            return "recurse"
+        self.update_recursion = True
         if not self.isValid():
-            return
+            return "not valid"
 
         game = self.game
         history_tail = game.historyGet()[self.history_index:]
-        
-        self.updateTimers(history_tail)        
-        packets = self.history2packets(history_tail, game.id, self.cache);
-        self.syncDatabase()
-        self.delayedActions()
-        if len(packets) > 0:
-            self.broadcast(packets)
-        self.tourneyEndTurn()
-        if self.isValid():
+
+        try:
+            self.updateTimers(history_tail)        
+            packets = self.history2packets(history_tail, game.id, self.cache);
+            self.syncDatabase()
+            self.delayedActions()
+            if len(packets) > 0:
+                self.broadcast(packets)
+            self.tourneyEndTurn()
+            if self.isValid():
+                self.cashGame_kickPlayerSittingOutTooLong(history_tail)
+                self.scheduleAutoDeal()
+        finally:
             self.historyReduce()
-            self.scheduleAutoDeal()
+            self.update_recursion = False
+        return "ok"
 
     def handReplay(self, client, hand):
         history = self.factory.loadHand(hand)
@@ -906,6 +938,7 @@ class PokerTable:
                 if client.removePlayer(self, serial):
                     self.seated2observer(client)
                     self.factory.leavePlayer(serial, game.id, self.currency_serial)
+                    self.factory.updateTableStats(game, len(self.observers), len(self.waiting))
                 else:
                     self.update()
             else:
@@ -931,6 +964,7 @@ class PokerTable:
             return
 
         self.factory.leavePlayer(serial, game.id, self.currency_serial)
+        self.factory.updateTableStats(game, len(self.observers), len(self.waiting))
 
         if self.serial2client.has_key(serial):
             self.seated2observer(self.serial2client[serial])
@@ -951,6 +985,7 @@ class PokerTable:
                 if client.removePlayer(self, serial):
                     self.seated2observer(client)
                     self.factory.leavePlayer(serial, game.id, self.currency_serial)
+                    self.factory.updateTableStats(game, len(self.observers), len(self.waiting))
                 else:
                     self.update()
             else:
@@ -984,6 +1019,7 @@ class PokerTable:
                 if client.removePlayer(self, serial):
                     self.seated2observer(client)
                     self.factory.leavePlayer(serial, game.id, self.currency_serial)
+                    self.factory.updateTableStats(game, len(self.observers), len(self.waiting))
                 else:
                     self.update()
             else:
@@ -1022,6 +1058,7 @@ class PokerTable:
             client.join(other_table)
         other_table.movePlayerTo(serial, money, sit_out)
         other_table.sendNewPlayerInformation(serial)
+        other_table.update()
         if self.factory.verbose:
             self.message("player %d moved from table %d to table %d" % ( serial, game.id, to_game_id ))
 
@@ -1050,6 +1087,8 @@ class PokerTable:
                                     auto_blind_ante = player.auto_blind_ante,
                                     wait_for = player.wait_for,
                                     seat = player.seat),
+            # the stats packet, generated by the UserStats package.
+            self.factory.getUserStatsLookup().getAttrsAsPacket(table = self, serial = serial),
             PacketPokerSeats(game_id = game.id, seats = game.seats()),
             PacketPokerPlayerChips(game_id = game.id,
                                    serial = serial,
@@ -1100,16 +1139,31 @@ class PokerTable:
             client.join(self);
             return True
 
+        # Next, test to see if we have reached the server-wide maximum for
+        # seated/observing players.
+        if not self.game.isSeated(client.getSerial()) and self.factory.joinedCountReachedMax():
+            self.error("joinPlayer: %d cannot join game %d because the server is full" %
+                       (serial, game.id))
+            client.sendPacketVerbose(PacketPokerError(game_id = game.id,
+                                                      serial = serial,
+                                                      other_type = PACKET_POKER_TABLE_JOIN,
+                                                      code = PacketPokerTableJoin.FULL,
+                                                      message = "This server has too many seated players and observers."))
+            return False
+
+        # Next, test to see if joining this table will cause the client to
+        # exceed the maximum permitted by the server.
         if len(client.tables) >= self.factory.simultaneous:
             if self.factory.verbose:
                 self.error("joinPlayer: %d seated at %d tables (max %d)" % ( serial, len(client.tables), self.factory.simultaneous ))
             return False
-        
+
         #
         # Player is now an observer, unless he is seated
         # at the table.
         #
         client.join(self)
+        self.factory.joinedCountIncrease()
         if not self.game.isSeated(client.getSerial()):
             self.observers.append(client)
         else:
@@ -1161,6 +1215,7 @@ class PokerTable:
         if amount > 0:
             client.setMoney(self, amount)
 
+        self.factory.updateTableStats(game, len(self.observers), len(self.waiting))
         return True
 
     def sitOutPlayer(self, client, serial):
@@ -1212,6 +1267,7 @@ class PokerTable:
         return client.sitPlayer(self, serial)
         
     def destroyPlayer(self, client, serial):
+        self.factory.joinedCountDecrease()
         if client in self.observers:
             self.observers.remove(client)
         else:
